@@ -1,8 +1,8 @@
 use std::{
-    collections::{BTreeMap, HashMap, HashSet},
+    collections::{BTreeMap, HashMap},
     net::SocketAddr,
     sync::Arc,
-    time::Duration,
+    time::{Duration, Instant},
 };
 
 use crate::{
@@ -220,8 +220,8 @@ pub struct WalletDump {
 
 // task that periodically pulls random coins to try to confirm
 async fn confirm_task(multi: MultiWallet, clients: HashMap<NetID, ValClient>) {
-    let mut pacer = smol::Timer::interval(Duration::from_millis(30000));
-    let sent = Arc::new(Mutex::new(HashSet::new()));
+    let mut pacer = smol::Timer::interval(Duration::from_millis(15000));
+    let sent = Arc::new(Mutex::new(HashMap::new()));
     loop {
         (&mut pacer).await;
         let possible_wallets = multi.list().collect::<Vec<_>>();
@@ -229,15 +229,16 @@ async fn confirm_task(multi: MultiWallet, clients: HashMap<NetID, ValClient>) {
             continue;
         }
         log::debug!("-- confirm loop sees {} wallets --", possible_wallets.len());
-        for wallet in possible_wallets {
-            let wallet = multi.get_wallet(&wallet);
+        for wallet_name in possible_wallets {
+            let wallet = multi.get_wallet(&wallet_name);
             match wallet {
                 Err(err) => {
                     log::error!("cannot read wallet: {}", err);
                 }
                 Ok(wallet) => {
                     let client = clients[&wallet.read().network()].clone();
-                    smolscale::spawn(confirm_one(wallet, client, sent.clone())).detach()
+                    smolscale::spawn(confirm_one(wallet_name, wallet, client, sent.clone()))
+                        .detach()
                 }
             }
         }
@@ -245,9 +246,10 @@ async fn confirm_task(multi: MultiWallet, clients: HashMap<NetID, ValClient>) {
 }
 
 async fn confirm_one(
+    wallet_name: String,
     wallet: AcidJson<WalletData>,
     client: ValClient,
-    sent: Arc<Mutex<HashSet<TxHash>>>,
+    sent: Arc<Mutex<HashMap<TxHash, Instant>>>,
 ) -> anyhow::Result<()> {
     let in_progress: BTreeMap<TxHash, Transaction> = wallet.read().tx_in_progress().clone();
     let in_progress: Vec<Transaction> = in_progress.values().cloned().collect();
@@ -257,7 +259,16 @@ async fn confirm_one(
     let snapshot = client.snapshot().await.context("cannot snapshot")?;
 
     for random_tx in in_progress {
-        if fastrand::f64() < 0.5 {
+        let should_send = {
+            let mut sent = sent.lock();
+            if let Some(res) = sent.get(&random_tx.hash_nosigs()) {
+                res.elapsed().as_secs_f64() > 120.0
+            } else {
+                sent.insert(random_tx.hash_nosigs(), Instant::now());
+                true
+            }
+        };
+        if should_send {
             log::warn!("transmit tx {}", random_tx.hash_nosigs());
             let result = snapshot
                 .get_raw()
@@ -271,43 +282,35 @@ async fn confirm_one(
                         random_tx.hash_nosigs(),
                         err
                     );
-                    // if fastrand::f64() < 0.1 && !err.to_string().contains("duplicate") {
-                    //     log::warn!(
-                    //         "retransmission of {} is very stuck, so reverting",
-                    //         random_tx.hash_nosigs()
-                    //     );
-                    //     wallet.write().force_revert_tx(random_tx.hash_nosigs())
-                    // }
                 }
                 Some(Ok(())) => {}
                 None => {
                     log::warn!("retransmission of {} timed out!", random_tx.hash_nosigs());
                 }
             }
-        } else {
-            // find some change output
-            let my_covhash = wallet.read().my_covenant().hash();
-            let change_indexes = random_tx
-                .outputs
-                .iter()
-                .enumerate()
-                .filter(|v| v.1.covhash == my_covhash)
-                .map(|v| v.0);
-            for change_idx in change_indexes {
-                let coin_id = random_tx.output_coinid(change_idx as u8);
-                log::trace!("confirm_one looking at {}", coin_id);
-                let cdh = snapshot
-                    .get_coin(random_tx.output_coinid(change_idx as u8))
-                    .await
-                    .context("cannot get coin")?;
-                if let Some(cdh) = cdh {
-                    log::debug!("confirmed {} at height {}", coin_id, cdh.height);
-                    wallet.write().insert_coin(coin_id, cdh);
-                    sent.lock().remove(&random_tx.hash_nosigs());
-                } else {
-                    log::debug!("{} not confirmed yet", coin_id);
-                    break;
-                }
+        }
+        // find some change output
+        let my_covhash = wallet.read().my_covenant().hash();
+        let change_indexes = random_tx
+            .outputs
+            .iter()
+            .enumerate()
+            .filter(|v| v.1.covhash == my_covhash)
+            .map(|v| v.0);
+        for change_idx in change_indexes {
+            let coin_id = random_tx.output_coinid(change_idx as u8);
+            log::trace!("confirm_one looking at {}", coin_id);
+            let cdh = snapshot
+                .get_coin(random_tx.output_coinid(change_idx as u8))
+                .await
+                .context("cannot get coin")?;
+            if let Some(cdh) = cdh {
+                log::debug!("confirmed {} at height {}", coin_id, cdh.height);
+                wallet.write().insert_coin(coin_id, cdh);
+                sent.lock().remove(&random_tx.hash_nosigs());
+            } else {
+                log::debug!("{} not confirmed yet", coin_id);
+                break;
             }
         }
     }
